@@ -1,6 +1,6 @@
 import contextlib
 import re
-from collections.abc import Iterable
+from collections.abc import Generator, Iterable
 from dataclasses import dataclass
 from typing import Any
 
@@ -47,7 +47,8 @@ class ImageName(object):
     def parse(s: str):
 
         # Registry absent version
-        matcher = re.compile(r"^(?P<components>[\w-]+(/[\w-]+)*)(:(?P<tag>[\w._-]+))?$")
+        matcher = re.compile(
+            r"^(?P<components>[a-z0-9]+((__?|-+)[a-z0-9]+)*(/[a-z0-9]+((__?|-+)[a-z0-9]+)*)*)(:(?P<tag>\w[\w.-]{0,127}))?$")
         match = matcher.match(s)
         if match:
             return ImageName(registry_host=None,
@@ -57,7 +58,7 @@ class ImageName(object):
 
         # Registry present version
         matcher = re.compile(
-            r"^(?:(?P<host>[\w.-]+)(?::(?P<port>\d+))?/)?(?P<components>[\w-]+(/[\w-]+)*)(:(?P<tag>[\w._-]+))?$")
+            r"^((?P<host>[a-zA-Z0-9.-]+)(:(?P<port>\d+))?/)?(?P<components>[a-z0-9]+((__?|-+)[a-z0-9]+)*(/[a-z0-9]+((__?|-+)[a-z0-9]+)*)*)(:(?P<tag>\w[\w.-]{0,127}))?$")
         match = matcher.match(s)
         if match:
             return ImageName(registry_host=match.group("host"),
@@ -176,24 +177,32 @@ def docker_run_detached(
     **kwargs,
 ) -> tuple[dict[str, Any], Any]:
     @contextlib.contextmanager
-    def managed_docker_run(client: docker.DockerClient, **kwargs) -> docker.models.containers.Container:
-        container_model = client.containers.run(**kwargs)
+    def managed_docker_run(
+        client: docker.DockerClient,
+        **kwargs,
+    ) -> Generator[docker.models.containers.Container, None, None]:
+        container_model = None
         try:
+            container_model = client.containers.run(**kwargs)
             yield container_model
+        except docker.errors.DockerException:
+            raise
         finally:
-            if container_model.status != "exited":
-                try:
-                    container_model.stop()
-                except docker.errors.DockerException:
-                    pass
-                try:
-                    container_model.wait()
-                except docker.errors.DockerException:
-                    pass
+            if container_model is None:
+                return
             try:
-                container_model.remove()
-            except docker.errors.DockerException:
-                pass
+                if container_model.status != "exited":
+                    try:
+                        container_model.stop()
+                    except docker.errors.DockerException:
+                        pass
+                container_model.wait()
+                try:
+                    container_model.remove()
+                except docker.errors.DockerException:
+                    pass
+            except Exception:
+                raise
 
     try:
         run_args = kwargs.copy()
